@@ -14,7 +14,7 @@ use utf8;
 ## use critic (Modules::RequireExplicitPackage)
 
 package Sys::OsPackage;
-$Sys::OsPackage::VERSION = '0.1.7';
+$Sys::OsPackage::VERSION = '0.2.0';
 use Config;
 use Carp qw(carp croak confess);
 use Sys::OsRelease;
@@ -118,7 +118,7 @@ sub class_or_obj
 
     # safety net: all-stop if we received an undef
     if (not defined $coo) {
-        confess "coo got undef from:".(join "|", caller 1);
+        confess "class_or_obj() got undef from: ".(join "|", caller 1);
     }
 
     # return the instance
@@ -238,6 +238,15 @@ sub quiet
     return deftrue($self->{_config}{quiet});
 }
 
+# read-only accessor for notest flag
+sub notest
+{
+    my ($class_or_obj) = @_;
+    my $self = class_or_obj($class_or_obj);
+
+    return deftrue($self->{_config}{notest});
+}
+
 # read/write accessor for system environment data
 # sysenv is the data collected about the system and commands
 sub sysenv
@@ -312,27 +321,27 @@ sub module_installed
 {
     my ($class_or_obj, $name, $value) = @_;
     my $self = class_or_obj($class_or_obj);
-
-    # if a value is provided then act as a write accessor to the module_installed flag for the module
-    if (defined $value) {
-        my $flag = $value ? 1 : 0;
-        $self->{module_installed}{$name} = $flag;
-        return $flag;
-    }
-
-    # short-circuit the search if we installed the module or already found it installed
-    return 1 if deftrue($self->{module_installed}{$name});
+    my $found = 0;
 
     # check each path element for the module
     my $modfile = join("/", split(/::/x, $name));
     foreach my $element (@INC) {
         my $filepath = "$element/$modfile.pm";
         if (-f $filepath) {
-            $self->{module_installed}{$name} = 1;
-            return 1;
+            $found = 1;
+            last;
         }
     }
-    return 0;
+
+    # if a value is provided, act as a write accessor to the module_installed flag for the module
+    # Set it to true if a true value was provided and the module was found in the @INC path.
+    if (defined $value) {
+        if ( $found and $value ) {
+            $self->{module_installed}{$name} = $found;
+        }
+    }
+
+    return $found;
 }
 
 # run an external command and capture its standard output
@@ -908,14 +917,18 @@ sub pkg_installed
 }
 
 # check if module is installed, and install it if not present
+# throws exception on failure
 sub install_module
 {
     my ($class_or_obj, $name) = @_;
     my $self = class_or_obj($class_or_obj);
-    $self->debug() and print STDERR "debug: install_module($name)\n";
+    $self->debug() and print STDERR "debug: install_module($name) begin\n";
+    my $result = $self->module_installed($name);
 
     # check if module is installed
-    if (not $self->module_installed($name)) {
+    if ($result) {
+        $self->debug() and print STDERR "debug: install_module($name) skip - already installed\n";
+    } else {
         # print header for module installation
         if (not $self->quiet()) {
             print  $self->text_green().('-' x 75)."\n";
@@ -923,23 +936,29 @@ sub install_module
         }
 
         # try first to install it with an OS package (root required)
-        my $done=0;
         if ($self->is_root()) {
             if ($self->module_package($name)) {
-                $self->module_installed($name, 1);
-                $done=1;
+                $result = $self->module_installed($name, 1);
             }
         }
 
         # try again with CPAN or CPANMinus if it wasn't installed by a package
-        if (not $done) {
-            my $cmd = (defined $self->sysenv("cpan") ? $self->sysenv("cpan") : $self->sysenv("cpanm"));
-            $self->run_cmd($cmd, $name)
+        if (not $result) {
+            my ($cmd, @test_param);
+            if (defined $self->sysenv("cpan")) {
+                $cmd = $self->sysenv("cpan");
+                $self->notest() and push @test_param, "-T";
+            } else {
+                $cmd = $self->sysenv("cpanm");
+                $self->notest() and push @test_param, "--notest";
+            }
+            $self->run_cmd($cmd, @test_param, $name)
                 or croak "failed to install $name module";
-            $self->module_installed($name, 1);
+            $result = $self->module_installed($name, 1);
         }
     }
-    return;
+    $self->debug() and print STDERR "debug: install_module($name) result=$result\n";
+    return $result;
 }
 
 # bootstrap CPAN-Minus in a subdirectory of the current directory
@@ -1044,11 +1063,11 @@ sub establish_cpan
         }
     }
 
-    # install dependencies for this tool
+    # install modules used by Sys::OsPackage or CPAN
     foreach my $dep (@{perlconf("module_deps")}) {
         $self->install_module($dep);
     }
-    return;
+    return 1;
 }
 
 1;
@@ -1063,7 +1082,7 @@ Sys::OsPackage - install OS packages and determine if CPAN modules are packaged
 
 =head1 VERSION
 
-version 0.1.7
+version 0.2.0
 
 =head1 SYNOPSIS
 
